package org.rfdump;

/**
 *
 * @author Boris Wolf
 * @version 09/17/2003
 *
 */

import javax.swing.table.*;

import java.util.ArrayList;
import java.util.HashMap;

public class RFIDTableModel extends AbstractTableModel {

    ArrayList memory = new ArrayList();
    HashMap writeBuffer;

	public RFIDTableModel(ArrayList memory, HashMap writeBuffer) {

        this.memory = memory;
        this.writeBuffer = writeBuffer;
    }

    public void append(String value) {

        this.memory.add(value);

        int column = (this.memory.size()-1)%8+1;
        int row = (this.memory.size()-((this.memory.size()-1)%8))/8;

        this.fireTableCellUpdated(row, column);
    }

    public void setPage(int adr, String value) {

        this.memory.set(adr, value);

        int column = adr%8+1;
        int row = ((adr-1)-((this.memory.size()-1)%8))/8;

        this.fireTableCellUpdated(row, column);
    }

    public String getColumnName(int col) {

        String header = "";

        switch (col) {

            case 0:
                header = "Adr";
                break;
            case 1:
                header = "0 / 8";
                break;
            case 2:
                header = "1 / 9";
                break;
            case 3:
                header = "2 / A";
                break;
            case 4:
                header = "3 / B";
                break;
            case 5:
                header = "4 / C";
                break;
            case 6:
                header = "5 / D";
                break;
            case 7:
                header = "6 / E";
                break;
            case 8:
                header = "7 / F";
                break;
        }

        return header;
    }

    public int getRowCount() { 
        return 32;
	}

    public int getColumnCount() { 
		return 9;
	}

    public Object getValueAt(int row, int col) {

        String value = "";

        if (col == 0) {

            if (row%2==1)
                row--;

            row /= 2;

            value = Integer.toHexString(row).toUpperCase();

        } else {

            int idx = row*8 + (col-1);

            if (idx <= memory.size()-1) {
                value = (String)memory.get(idx);
            } else {
                value = "--------";
            }
        }

        return value;
    }

    public boolean isCellEditable(int row, int col) {

        if (col>0)
    		return true;
        else
            return false;
	}

    public void setValueAt(Object value, int row, int col) {

        int PAGE_LENGTH = 16;

        if (this.memory.size()>0)
            PAGE_LENGTH = ((String)this.memory.get(0)).length();

        String strValue = (String)value;

        if (strValue.length() > PAGE_LENGTH) {

            strValue = strValue.substring(0, PAGE_LENGTH);

        } else if (strValue.length() < PAGE_LENGTH) {

            String padding = "";

            for (int i=0; i<PAGE_LENGTH-strValue.length(); i++)
                padding += "0";

            strValue = padding + strValue;
        }

        int adr = row*8 + (col-1);
        String hexAdr = Integer.toHexString(adr);

        if (hexAdr.length() == 1)
            hexAdr = "0" + hexAdr;

        memory.set(adr, strValue);
        writeBuffer.put(hexAdr, strValue);
    }
}

